package main

import (
	"cafe/api"
	"cafe/config"
	"cafe/service"
	"cafe/user"
	"fmt"

	"github.com/gin-gonic/gin"
	"github.com/sirupsen/logrus"
)

func main() {
	gin.SetMode(gin.ReleaseMode)

	config.Cafe.Database.Initialize(config.StorageDir)
	config.Cafe.Database.MigrateHelper(service.Table{}, "table")
	config.Cafe.Database.MigrateHelper(service.Order{}, "order")
	config.Cafe.Database.MigrateHelper(service.OrderItem{}, "orderItem")
	config.Cafe.Database.MigrateHelper(service.Bill{}, "bill")
	config.Cafe.Database.MigrateHelper(service.BillItem{}, "billItem")
	config.Cafe.Database.MigrateHelper(user.User{}, "user")

	a := api.Api{}
	service.Initialize()
	a.Hub.Initialize()

	a.Router = gin.New()
	a.SetMiddlewares()
	a.HandleStaticFiles()
	a.SetupSwagger()
	a.SetupRouter()
	logrus.WithField("port", config.Cafe.Port).Info("Server running")
	err := a.Router.Run(fmt.Sprintf(":%d", config.Cafe.Port))
	if err != nil {
		logrus.WithField("error", err).Fatal("Cannot start server")
	}
}
